package hibtest.envers;

import hibtest.AbstractTest;

import java.util.Date;

import org.hibernate.envers.AuditReader;
import org.hibernate.envers.AuditReaderFactory;
import org.testng.Assert;
import org.testng.annotations.Test;

import envers.AudChild;
import envers.AudParent;

public class EnversTest extends AbstractTest {
    
    @Test
    public void persistAndChange() {
        AudParent p = new AudParent("p1");
        AudChild c = new AudChild(p, "c1");
        
        
        entityManager.persist( c );
        p.getChildren().add( c );
        entityManager.persist( p );
        
        
        end();
        
        begin();
        
        AudParent same = entityManager.find( AudParent.class, p.getId() );
        
        Assert.assertNotNull( same );
        Assert.assertTrue( same.getChildren().size() == 1 );
        
        Number rev = getAuditReader().getRevisionNumberForDate( new Date() );
        Assert.assertNotNull( rev );
        
        AudParent versioned = getAuditReader().find( AudParent.class, p.getId(), rev );
        Assert.assertNotNull( versioned );
        Assert.assertEquals( versioned.getName(), "p1" );
        Assert.assertTrue( versioned.getChildren().size() == 1 );
        
        end();
        
        
        begin();
        AudChild c2 = entityManager.find( AudChild.class, c.getId() );
        
    }

    private AuditReader getAuditReader() {
        return AuditReaderFactory.get( entityManager );
    }

}
